@@ -1,3 +1,7 @@
+(* TEST
+   include testing
+*)
+
 let f = Format.sprintf "[%i]";;
 print_endline (f 1);;
 print_endline (f 2);;
